% aktool
%
% 18 июля 2020 г.

НАЗВАНИЕ
========

aktool - консольная утилита для шифрования и контроля целостности данных.

СИНТАКСИС
=========

Синтаксис выполнения программы

aktool **команда** \[ опции \] \[ файл или каталог \] \...


ОПИСАНИЕ
========

Утилита предназначена для проведения различных криптографических
преобразований таких, как шифрование и контроль целостности данных,
выработка и проверка электронной подписи, генерация криптографических
ключей и паролей, создание и верификация сертификатов открытых ключей.

Утилита предоставляет интерфейс командной строки для доступа к
преобразованиям, реализуемым библиотекой *libakrypt* в
соответствии с «Принципами разработки и  модернизации шифровальных
(криптографических) средств защиты информации» для средств
класса КС3.

Каждая команда определяет отдельный класс криптографических
преобразований и функциональных возможностей. В настоящее время
доступны следующие команды:

**a, asn1parse**
: Декодирование и печать ASN.1 данных

**i, icode**
: Вычисление контрольной суммы или имитовставки для одного или
нескольких файлов

**k, key**
: Управление ключевой информацией

**s, show**
: Вывод служебной и справочной информации

**test**
: Тестирование корректной работы и эксплуатационных характеристик утилиты

Получить подробную информацию о доступных опциях и параметрах любой
команды можно с помощью вызова


    aktool команда --help


УПРАВЛЕНИЕ КЛЮЧЕВОЙ ИНФОРМАЦИЕЙ
===============================

## Общее описание

Команда `key` (короткая форма - `k`) позволяет вырабатывать ключи
симметричных и асимметричных криптографических преобразований для
дальнейшего использования в алгоритмах шифрования, контроля целостности,
процедурах выработки и проверки электронной подписи и т.п.

Команда `key` позволяет сохранять секретные ключи в файлы или
ключевое хранилище в контейнере, формат которого является открытым
и распространяется вместе с документацией к библиотеке *libakrypt*.
Перед сохранением секретного ключа пользователю будет предложено
ввести пароль, используемый для шифрования созданного ключа.

Открытые ключи могут сохраняться как в формате запроса на сертификат,
так и в формате собственно сертификата в соответствии с рекомендациями
Р 1323565.1.23-2018. Также команда `key` позволяет просматривать и
удалять созданные ранее ключи.


## Опции команды key

\-a, \--algorithm \<ni\>
: Опция позволяет указать алгоритм, задаваемый параметром `ni`
(name or identifier), для которого будет вырабатываться секретый ключ.
Если это секретный ключ асимметричного преобразования, то одновременно
будет выработан и соответствующий открытый ключ.

\--curve \<ni\>
: Опция позволяет в явном виде указать имя или идентификатор эллиптической
кривой, используемой в асимметричных криптографических механизмах
(электронной подписи, асимметричного шифрования и т.п.).
Для секретных ключей симметричных механизмов значение опции игнорируется.

\-days \<value\>
: Опция позволяет установить количество дней действия ключа.
По умолчанию это значение равно 365. Опция может быть применима ко всем
создаваемым ключам.

\-l, \--label \<text\>
: Опция позволяет установить для создаваемого секретного ключа
метку, т.е. человекочитаемое имя, описанние, алиас, комментарий и т.п.,
которая может быть использована пользователем по своему усмотрению,
например, для указания области действия ключа или подсказки для
запоминания пароля.

\-n, \--new
: Опция, запускающая процедуру выработки нового ключа или
ключевой пары (для асимметричных криптографических механизмов).

\-o, \--outkey \<file\>
: Опция определяет имя файла, используемого для сохранения созданного
секретного ключа. Ключ сохраняется в формате контейнера библиотеки *libakrypt*.

\--password \<pass\>
: Опция позволяет определить пароль, используемый для
для шифрования создаваемого ключа. Если опция не определена, то утилита
предлагает пользователю ввести пароль с консоли.

\--pubkey \<file\>
: Опция определяет имя файла, используемого для сохранения созданного
открытого ключа. Данная опция имеет смысл только при создании ключей
асимметричных криптографических механизмов.

\-to \<format\>
: Опция определяет формат, в котором будет сохранен созданый открытый ключ.
Допустимыми значениями яляются `der`, `pem` и `certificate`.
Если используются значения `der` или `pem`, то открытый ключ сохраняется
в виде запроса на сертификат, соответственно, в кодировке `der` или `pem`.
Если задано значение `certificate`, то созданный открытый ключ сохраняется
в виде самоподписанного корневого сертификата в кодировке `pem`.


Следующие опции имеет смысл использовать только при создании сертификатов
открытых ключей (при использовании опции `--new` совместно
со значением `--to certificate`, а также при использовании опции `--cert`).


\--ca \<value\>
: Опция имеет смысл при создании самоподписанных сертификатов, принимаемые
значения: `true` или `false`.
Истинное значение булевой переменной `cA` указывает, что
помощью данного сертификата можно порождать цепочки сертификации.
Значение `false` означает, что данный самоподписанный
сертификат открытого ключа не должен использоваться для проверки подписи.


\--pathlen \<value\>
: Опция имеет смысл при создании самоподписанных сертификатов
в случае, когда установлено значение `--ca=true`.
В этом случае `value` задает максимальное возможное количество сертификатов
(длину цепочки) между самоподписанным сертификатом и последним элементом
цепочки. Значение `--pathlen=0` указывает, что промежуточных
сертификатов нет и цепочка содержит только один сертификат низлежащего уровня.


\--digitalSignature
: Опция указывает, что сертификат может быть использован
для проверки электронной подписи, за исключением
проверки подписей под сертификатами, списками отозванных сертификатов и
предоставления данных службе  аннулирования сертификатов.

\--contentCommitment
: Опция указывает, что сертификат может быть использован
для проверки электронных подписей, используемых для предоставления данных службе
аннулирования сертификатов.

\--keyEncipherment
: Опция указывает, что сертификат может быть использован
для шифрования закрытых или секретных ключей, т.е. выступает в роли
транспортного ключа. В библиотеке `libakrypt` данные сертификаты
используются для открытых ключей, используемых в гибридной схеме шифрования.

\--dataEncipherment
: Опция указывает, что сертификат может быть использован для
непосредственного шифрования необработанных пользовательских данных
без использования симметричного шифрования.

\--keyAgreement
: Опция указывает, что сертификат может быть использован для
аутентификации сторон в протоколах выработки общего ключа.

\--keyCertSign
: Опция указывает, что сертификат может быть использован
для проверки подписей под сертификатами.

\--cRLSign
: Опция указывает, что сертификат может быть использован
для проверки подписей под списками отозванных сертификатов.


## Примеры выработки ключевой информации


### Ключи симметричных криптографических алгоритмов


Создать новый ключ выработки имитовставки для алгоритма `hmac256-streebog256`
можно с помощью следующего вызова. Создаваемый ключ будет помечен
меткой *used for testing purpose* и сохранен в файл `hmac.k`.


    aktool k -na hmac-streebog256 -o hmac.k -l "used for testing purpose"


Вывести содержимое созданного ключевого контейнера в формате ASN.1 дерева
можно с помощью следующего вызова.


    aktool a hmac.k



### Ключевые пары и сертификаты открытых ключей


КОНТРОЛЬ ЦЕЛОСТНОCТИ
====================

## Общее описание

Команда `icode` позволяет вычислять контрольные суммы или имитовставки
для одного или нескольких файлов. В случае указания в командной строке
имени каталога контрольные суммы или имитовставки будут подсчитаны для
всех доступных для чтения файлов в указанном каталоге.

Перечень алгоритмов бесключевого хеширования, используемых для
вычисления контрольной суммы, может быть получен с помощью
следующего вызова.


    aktool s --oid hash


Аналогично, перечень доступных алгоритмов имитозащиты
может быть получен с помощью следующего вызова.


    aktool s --oid mac


В отличие от алгоритмов вычисления контрольной суммы, при вычислении имитовставки в
обязательном порядке должен быть использован секретный ключ. Данный
ключ может быть:

 - предварительно создан пользователем,
 - выработан в процессе работы из заданного пользователем пароля,
 - указан в явном виде.

Способ передачи ключа в утилиту *aktool* определяется ее опциями.

Поскольку некоторые алгоритмы вычисления имитовставки накладывают
ограничения на объем информации, обрабатываемой на одном ключе, то для
таких алгоритмов вычисление имитоставок от очень больших
файлов может приводить к ошибке. В этом случае вместо имитовставки
выводится значение `skipped`. Если Вам необходимо вычислять
имитовставки файлов с произвольной, сколь угодно большой длиной, то
используйте следующие алгоритмы: `hmac-streebog256`, `hmac-streebog512`.

## Опции команды icode

\-a, \--algorithm \<ni\>
: Опция позволяет указать алгоритм, задаваемый параметром `ni`
(name or identifier), который будет использован для вычисления
контрольной суммы или имитовставки. В качестве параметра может выступать
имя алгоритма, например `streebog512`, или идентификатор
алгоритма \- последовательность целых чисел, разделенный точками.
В случае, когда опция не определена, для вычисления контрольной суммы
используется бесключевая функция хеширования `streebog256`,
регламентированная национальным стандартом ГОСТ Р 34.11-2012.
В случае выработки имитовставки, если секретный ключ указывается
с помощью опции `--key`, данная опция может быть опущена.

\-c, \--check \<file\>
: Опция позволяет проверить контрольные суммы или имитовставки для
одного или нескольких файлов. Данные суммы должны быть вычислены
заранее и сохранены в файле `file`. Если при проверке используется
алгоритм, отличный от установленного по-умолчанию, необходимо указать
его имя или идентификатор с помощью опции `-a`. Также, при необходимости,
нужно указать используемый ключ.


\--hexkey \<string\>
: Опция в явном виде указывает ключ, используемый для выработки
имитовставки. Ключ должен представлять собой последовательность
шестнадцатеричных символов, не более 128 символов длиной, что
регламентируется рекомендациями Р 50.1.113-2016. Использование
данной опции не является безопасным.


\--key \<file\>
: Опция устанавливает имя файла с секретным ключом,
используемым для вычисления или проверки имитовставки. Если алгоритм,
для которого выработан ключ, отличается от алгоритма, указанного с помощью
опции `-a`, то будет использован алгоритм секретного ключа.


\-o, \--output \<file\>
: Опция определяет имя файла, в который записываются вычисленные
значения контрольных сумм или имитовставок. Данный файл может быть
указан в качестве параметра опции `-c`.


\-p
: Опция указывает, что для выработки секретного ключа, используемого
при вычислении или проверке имитовставки от одного или нескольких
файлов, должен использоваться задаваемый пользователем пароль. Если
опция `password` не опредена, то утилита *aktool* предлагает пользователю
ввести пароль с консоли.


\--password \<pass\>
: Опция в явном виде указывает, что для выработки секретного ключа
должен использоваться пароль, определяемый последовательностью
символов `pass` в командной строке. Использование данной опции не
является безопасным.


\-r, --recursive
: Опция указывает, что при вычислении контрольных сумм или
имитовставок, должна выполняться рекурсивная процедура обхода
текущего и всех вложенных каталогов. Опция имеет смысл только в том
случае, когда в аргументах вызова утилиты указано имя каталога.


\--reverse-order
: Опция указывает, что все выводимые и вводимые последовательности
октетов, должны обрабатываться в обратном порядке. Использование
данной опции целесообразно для вывода результатов в `big endian`
порядке следования октетов.


\--salt \<hexstring\>
: При генерации ключа из пароля используется криптографическая функция
PBKDF2, инициализируемая двумя значениями \- паролем и
инициализационным вектором. Данная опция позволяет определить
значение инициализационного вектора, передаваемого как
последовательность шестнадцатеричных символов. Если количество
шестнадцатеричных символов менее величины, определяемой опцией `--salt-len`,
то инициализационный вектор добавляется нулями в старших разрядах.

\--salt-len \<int\>
: Опция позволяет изменить количество октетов в инициализационном
векторе, используемом в криптографической функции PBKDF2. Значение
параметра должно принимать значение от 8 до 32-х. По-умолчанию,
используется знаение 16. В случае, если инициализационный вектор,
передаваемый в программу с помощью опции `--salt` имеет меньшую
длину, он дополняется нулями в старших октетах.


\--tag
: Опция указывает, что значения контрольных сумм или имитовставок
должны выводиться в формате, принятом в операционных системах
семейства BSD. В таком формате указывается не только имя файла и его
контрольная сумма, но и алгоритм, с помощью которого данная сумма
была подсчитана. Это может быть востребовано при проверке, поскольку
у пользователя пропадает необходимость в явном виде указывать
используемый алгоритм выработки контрольных сумм или имитовставок.


\-t, --template \<mask\>
: Опция указывает шаблон (маску) поиска файлов, к которым применяется процедура
вычисления контрольной суммы или имитовставки.

 Следующие опции имеет смысл применять только при проверке контрольных сумм или имитовставок.

\--dont-show-stat
: Опция запрещает вывод в консоль статистической информации об общем
количестве проверенных файлов, количестве успешных или неуспешных
проверок и т.п.

\--ignore-errors
: Опция запрещает останавливать процесс проверки контрольных сумм или
имитовставок в случае возникновения ошибок доступа или чтения
файлов, отсутствия файлов, содержащихся в списке и т.п.

\--quiet
: Опция запрещает вывод символов `Ok` и `Wrong` после проверки
контрольной суммы.

\--status
: Опция запрещает вывод какой-либо информации при проверке контрольных
сумм. Результат проверки возвращается в коде возврата программы
(любое отличное от нуля значение сигнализирует об ошибке).

## Примеры контроля целостности

Приведенный ниже вызов утилиты вычисляет контрольную сумму файла
`file.txt` с помощью установленного по умолчанию алгоритма `streebog256`
и помещает результат вычислений в файл `result.txt`.


    aktool i file.txt -o result.txt


Следующий вызов позволяет вычислить код целостности всех файлов,
удовлетворяющих шаблону `\*.t??` (файлы, имеющие расширение из трех
символов, начинающееся с символа t) в текущем каталоге (на это указывает
символ `.`), а также во всех доступных вложенных каталогах. Для
вычисления кода целостности используется алгоритм выработки имитовставки
ГОСТ Р 34.13-2015 в основе которого лежит блочный шифр `kuznechik`. Для
имитозащиты файлов используется ключ, вырабатываемый из пароля, который
должен быть введен пользователем.


    aktool i -rt "*.t??" -a cmac-kuznechik -p .


Следующий вызов позволяет проверить контрольные суммы, содержащиеся в
выработанном ранее файле`result.txt`. При проверке используется
установленный по-умолчанию алгоритм `streebog256`. Если файл
`result.txt` формировался с использованием опции `--tag`, то для
проверки будет использован алгоритм, указанный в `result.txt`.


    aktool i -c result.txt

Следующий пара вызовов позволяет вычислить и проверить имитовставки
группы файлов, задаваемых шифлоном `*.txt`.
Вырабатываемые значения помещаются в файл `mac.txt`.
При выработке имитоставки используется алгоритм, определяемый
секретным ключом (в данном случае -- `hmac-streebog256`). Флаг `--tag`
позволяет в явном виде сохранить в файл `mac.txt` имя используемого
алгоритма выработки имитовставки.


    aktool i -k hmac.k  --tag  *.txt -o mac.txt
    aktool i -c mac.txt -k hmac.k


Последний вызов позволяет проверить значения имитовставок с помощью
явно указанного алгоритма `hmac-streebog256`. Ключ имитозащиты
вырабатывается из пароля, указанного в явном виде пользователем
в командной строке.


    aktool i -c result.txt -a hmac-streebog256 --password aQ13jzUl


## Совместимость с другими программными средствами

Известны две общедоступные утилиты, позволяющие вычислять контрольные
суммы с помощью отечественных алгоритмов хеширования. Этими утилитами
являются:

 - *gost12sum* ( https://github.com/gost-engine/engine ), которая входит в
состав дополнительного модуля (engine) для хорошо известной программы
*openssl*,

 - *rhash* ( https://github.com/rhash/RHash ).

Следующая последовательность команд позволяет продемонстрировать
эквивалентность работы всех трех утилит для алгоритма хеширования
`streebog256`.


    echo -n hello > test.file
    aktool i test.file
    3fb0700a41ce6e41413ba764f98bf2135ba6ded516bea2fae8429cc5bdd46d6d test.file


    rhash -G test.file
    3fb0700a41ce6e41413ba764f98bf2135ba6ded516bea2fae8429cc5bdd46d6d test.file


    gost12sum test.file
    3fb0700a41ce6e41413ba764f98bf2135ba6ded516bea2fae8429cc5bdd46d6d test.file


Аналогично, для алгоритма хеширования `streebog512`, запуск следующих
команд также позволит получить одинаковое значение хэш-кода размером 512 бит.


    gost12sum -l test.file
    rhash --gost12-512 test.file
    aktool i -a streebog512 test.file


Сама программа *openssl* также позволяет вычислять контрольные суммы,
например, следующий вызов позволит получить тот же результат, но в
другом формате


    openssl dgst -md_gost12_256 test.file > result.txt
    cat result
    md_gost12_256(test.file)=
               3fb0700a41ce6e41413ba764f98bf2135ba6ded516bea2fae8429cc5bdd46d6d


Утилита `aktool` может не только вырабатывать контрольные суммы в
таком же формате,


    aktool i --tag test.file
    streebog256 (test.file) =
               3fb0700a41ce6e41413ba764f98bf2135ba6ded516bea2fae8429cc5bdd46d6d

но и проверять файлы, подготовленные программой *openssl*, используя
единый синтаксис командной строки.


    aktool i -c result.txt



При имитозащите данных утилита *aktool* совместима с программой *openssl*
при использовании явно заданных ключей. Следующие вызовы используют
для вычисления имитовставки функцию `hmac-streebog256` и позволяют
получить одинаковый результат


    openssl dgst -md_gost12_256 -mac hmac -macopt hexkey:acabdcab0137
    test.file HMAC-md_gost12_256(test.file)=
               31d56cc082c4cd57955119f4cde662b4554f867fcc08ee4d59ba32d0e604180d


    aktool i -a hmac-streebog256 --hexkey acabdcab0137 --tag test.file
    hmac-streebog256 (test.file) =
               31d56cc082c4cd57955119f4cde662b4554f867fcc08ee4d59ba32d0e604180d


Отметим, что в данном случае секретный ключ, используемый для вычисления
имитовставки, задается явно, в виде последовательности шестнадцатеричных
символов. Максимальная длина последовательности не должна превышать 64
октета (128 шестнадцатеричных символа). Для последовательностей меньшей
длины старшие октеты заполняются нулями. Аналогичная совместимость
поддерживается при использовании функции `hmac-streebog512`.


РАЗБОР ДАННЫХ В ФОРМАТЕ ASN.1
=============================

Команда `asn1parse` позволяет декодировать и выводить в консоль данные,
закодированные в формате ASN.1 дерева. В настоящее время
поддерживается разбор чистых `der`-последовательностей, а также
der-последовательностей, закодированных в кодировке base64
(формат сертификатов `pem`).

## Опции команды asn1parse

\--convert
: Опция позволяет выполнить конвертацию ASN.1 данных между поддерживаемыми форматами,
т.е. перевести данные из `der`-формата в `pem`-формат и наоборот.
Преобразование, изменение или удаление данных из ASN.1 дерева при конвертации не производится.

\--split
: Опция позволяет разделить ASN.1 дерево на одну или несколько ветвей первого уровня.
После разделения каждая ветвь образует новое ASN.1 дерево и помещается в отдельный файл.
Опция может применяться для разделения коллекций сертификатов открытых ключей на
отдельные файлы.

\-o, \--output \<файл\>
: Опция позволяет в явном виде определить
имя файла в который будут помещены конвертированные или объединенные данные.
Данную опцию имеет смысл использовать совместно с опциями `--convert` и `--join`.

\--to \<формат\>
: Опция в явном виде указывает в каком формате должны быть сохранены данные.
Доступными значениями поля формат являются: `der` и `pem`.

\--pem \<тип\>
: Опция в явном виде позволяет определить заголовок строки
в `pem`-файле, определяющий тип содержащихся далее данных.
Данную опцию имеет смысл использовать совместно с указаним опции  `--to=pem`.
При значении `--to=der` значение опции игнорируется.

## Примеры использования

Рассмотрим запрос на создание сертифката открытого ключа,
записанный в pem кодировке в файле `request_edw.pem`. Данный запрос
содержится в качестве примера в рекомендациях Р 1323565.1.023-2018, раздел А.2.2.1.


    cat request_edw.pem
    -----BEGIN CERTIFICATE-----
    MIHKMHkCAQAwEjEQMA4GA1UEAxMHRXhhbXBsZTBeMBcGCCqFAwcBAQEBMAsGCSqF
    AwcBAgEBAQNDAARAdCeV1L7ohN3yhQ/sA+o/rxhE4B2dpgtkUJOlXibfw5l49ZbP
    TU0MbPHRiUPZRJPRa57AoW1RLS4SfMRpGmMY4qAAMAoGCCqFAwcBAQMCA0EAG9wq
    Exdnm2YjL2PqFv98ZMyqua2FX8bhgJFmHbedSBIdDh2lvjR8bxtSVseurCAK1krH
    em9bOg4Jcxjnrm7naQ==
    -----END CERTIFICATE-----

Декодирование данного файла сможет быть выполнено с помощью следующего вызова.


    aktool a request_edw.pem


В результате декодирования в консоль должна быть выведена следующая
иерархическая структура.


    ┌SEQUENCE┐
             ├SEQUENCE┐
             │        ├INTEGER 0x0
             │        ├SEQUENCE┐
             │        │        └SET┐
             │        │            └SEQUENCE┐
             │        │                     ├OBJECT IDENTIFIER 2.5.4.3
             │        │                     └PRINTABLE STRING Example
             │        ├SEQUENCE┐
             │        │        ├SEQUENCE┐
             │        │        │        ├OBJECT IDENTIFIER 1.2.643.7.1.1.1.1
             │        │        │        └SEQUENCE┐
             │        │        │                 └OBJECT IDENTIFIER 1.2.643.7.1.2.1.1.1
             │        │        └BIT STRING
             │        │           04 40 74 27 95 D4 BE E8 84 DD F2 85 0F EC 03 EA
             │        │           3F AF 18 44 E0 1D 9D A6 0B 64 50 93 A5 5E 26 DF
             │        │           C3 99 78 F5 96 CF 4D 4D 0C 6C F1 D1 89 43 D9 44
             │        │           93 D1 6B 9E C0 A1 6D 51 2D 2E 12 7C C4 69 1A 63
             │        │           18 E2
             │        └[0]┐
             │             (null)
             ├SEQUENCE┐
             │        └OBJECT IDENTIFIER 1.2.643.7.1.1.3.2
             └BIT STRING
               1B DC 2A 13 17 67 9B 66 23 2F 63 EA 16 FF 7C 64
               CC AA B9 AD 85 5F C6 E1 80 91 66 1D B7 9D 48 12
               1D 0E 1D A5 BE 34 7C 6F 1B 52 56 C7 AE AC 20 0A
               D6 4A C7 7A 6F 5B 3A 0E 09 73 18 E7 AE 6E E7 69


Следующий вызов демонстрирует
пример конвертирования ASN.1 дерева, сохраненного в виде
`der`-последовательности, в формат `pem`.


    aktool a --convert request_edw.p10 --to pem --pem request -o request_edw.pem


Следующий вызов
приводит пример того, как можно разделить коллекцию сертификатов, содержащуюся в файле
`ca-bundle.crt`, на отдельные файлы.


    aktool a --split ca-bundle.crt --to pem --pem certificate



ВЫВОД СЛУЖЕБНОЙ И ВСПОМОГАТЕЛЬНОЙ ИНФОРМАЦИИ
============================================

Команда `show` позволяет получить информацию о доступных
криптографических преобразованиях, значениях технических и
криптографических характеристик, параметрах библиотеки по-умолчанию. В
настоящий момент доступны следующие опции.

## Опции команды show

\--curve
: Опция позволяет вывести в консоль параметры эллиптической кривой с заданным именем
или идентификатором.

\--engines
: Опция позволяет вывести список всех типов криптографических
механизмов, которые могут быть использованы в приложениях библиотеки
*libakrypt*. К таким типам, например, относятся блочные шифры или
алгоритмы выработки имитовставки.

\--modes
: Опция позволяет вывести список всех режимов криптографических
преобразований, которыми могут быть охарактеризованы
криптографические преобразования. При этом одному криптографическому
механизму может соответствовать несколько режимов. Например, функция
хеширования имеет режим `algorithm`, а ее параметры `kbox params`.

\--oids
: Опция позволяет вывести список всех доступных криптографических
механизмов. Каждый механизм идентифицируется свои именем, которое
используется для его указания пользователем и/или
последовательностью чисел, разделенных точками. Последняя
последовательность называется Object IDentifier (OID) и используется
при автоматизированной обработке данных.

\--oid
: Опция принимает в качестве параметра произвольную строку символов
`eni` (engine, name or identifier) и выводит все доступные
криптографические механизмы, в именах или идентификаторах которых
содержится указанная строка.

\--options
: Опция выводит перечень всех криптографических параметров библиотеки,
а также их значений. К указанным параметрам относятся, например,
число блоков информации, которые могут быть зашифрованы на одном
ключе, или число итераций криптографической функции PBKDF2,
используемой для развертки ключа из пароля.

\--without-caption
: Опция запрещает печать заголовка, расшифровывающего названия
выводимых параметров и их значений.


## Примеры использования

Наиболее употребляемым вариантом вызова команды `show` является вызов


    aktool s --oids


В результате данного вызова будет выведена информация
о всех доступных библиотеке *libakrypt* криптографических преобразованиях.
Например, среди прочих, будет выведена  информация об алгоритме
блочного шифрования Кузнечик, см. ГОСТ Р 34.12-2015.


     N  oid                 name(s)          engine            mode
    --------------------------------------------------------------------------
     9  1.2.643.7.1.1.5.2   kuznechik        block cipher      algorithm
                            kuznyechik
                            grasshopper


В данной таблице указаны

 - порядковый номер криптографического механизма,
 - идентификатор (последовательность цифр, разделенных точками),
 - перечень имен, с помощью которых пользователь может именовать
   криптографический механизм,
 - тип криптографического механизма,
 - режим использования.

Выборочный поиск можно осуществить с помощью опции `--oid`. Например,
следующий вызов позволяет найти все идентификаторы, имена которых
регламентированы Техническим комитетом *Криптографическая защита информации* (ТК26).


    aktool s --oid tc26


Получить параметры эллиптической кривой можно с помощью следующего вызова.


    aktool s --curve cspa


После чего, в консоль будет выведена следующая информация.


    curve: id-tc26-gost-3410-2012-256-paramSetB
           id-rfc4357-gost-3410-2001-paramSetA
           id-rfc4357-2001dh-paramSet
           cspdh
           cspa

    oid:   1.2.643.7.1.2.1.1.2
           1.2.643.2.2.35.1
           1.2.643.2.2.36.0

    parameters:
      a =  fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffd94
      b =  00000000000000000000000000000000000000000000000000000000000000a6
      p =  fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffd97
      q =  ffffffffffffffffffffffffffffffff6c611070995ad10045841b09b761b893
      c =  1 [cofactor]

    point:
     px =  0000000000000000000000000000000000000000000000000000000000000001
     py =  8d91e471e0989cda27df505a453f2b7635294f2ddf23e3b122acc99c9e9f1e14


Перечень всех известных библиотеке идентификаторов эллиптических кривых
может быть получен с помощью следующего вызова.


    aktool s --oid curve


ТЕСТИРОВАНИЕ ПРОГРАММЫ
======================

Команда `test` позволяет протестировать корректность выполняемых криптографических
механизмов, а также оценить эксплуатационные характеристики утилиты.

## Опции команды test

\--dynamic
: Опция позволяет выполнить так называемый динамический контроль, то есть
проверить корректность работы всех криптографических механизмов, реализованных библиотекой `libakrypt`,
на фиксированных контрольных примерах.


## Примеры использования

Для выполнения динамического контроля можно выполнить следующий вызов.


    aktool test --dynamic --audit 2 --audit-file stderr



ОПЦИИ, ОБЩИЕ ДЛЯ ВСЕХ КОМАНД
============================

\--audit
: Опция позволяет указать уровень аудита, т.е количество выводимых сообщений в процессе работе
программы. Если уровень аудита равен нулю, то выводятся только сообщения об ошибках.
При уровне, равным единице, выводятся обязательные сообщения, указанные в Р 1323565.1.012-2017,
при двойке выводятся диагностические и информационные сообщения.
Заметим, что уровень аудита, установленный в конфигурационном файле библиотеки,
не может быть снижен.

\--audit-file
: Опция позволяет определить файл, в который будут помещаться сообщения аудита.
В случае, если аргумент опции принимает значение `stderr`,
то будет использован системный канал вывода ошибок.

\--dont-use-colors
: По умолчанию для акцентирования внимания пользователя утилита использует цветовое выделение
при выводе некоторых данных - кодов ошибок, сообщений об автоматическом
декодировании ASN.1 деревьев и т.п. Данная опция отключает цветовое выделение.

\--openssl-style
: Опция должна использоваться для принудительного использования
блочных шифров и производных от них криптографических преобразований в соответствии с
форматами представления данных, принятыми в библиотеке openssl.
Данные форматы являются общепринятыми, но, к сожалению, не соответствуют
контрольным примерам из ГОСТ Р 34.12-2015 и ГОСТ Р 34.13-2015.


## Примеры использования

Следующий вызов содержит
пример перенаправления вывода утилиты *aktool* с запрещением использования цвета
на вход программы просмотра *less* (запрет использования цвета убирает управляющие команды,
которые не могут быть обработаны утилитой просмотра).


    aktool a 0408435EB90E5C8796A160E69E4BFAC453435D1D.cer --dont-use-colors | less



СТАНДАРТЫ
=========

## Национальные стандарты Российской Федерации
 - ГОСТ Р 34.10-2012 Процессы формирования и проверки электронной подписи,
 - ГОСТ Р 34.11-2012 Функция хэширования,
 - ГОСТ Р 34.12-2015 Блочные шифры,
 - ГОСТ Р 34.13-2015 Режимы работы блочных шифров.

## Рекомендации по стандартизации Росстандарта России
 - Р 50.1.111-2016 Парольная защита ключевой информации,
 - Р 50.1.113-2016 Криптографические алгоритмы, сопутствующие применению алгоритмов
   электронной цифровой подписи и функции хэширования,
 - Р 1323565.1.012-2017 Принципы разработки и модернизации шифровальных (криптографических) средств защиты информации,
 - Р 1323565.1.023-2018 Использование алгоритмов ГОСТ Р 34.10-2012, ГОСТ Р 34.11-2012 в сертификате,
   списке аннулированных сертификатов (CRL) и запросе на сертификат PKCS #10 инфраструктуры открытых ключей X.509,
 - Р 1323565.1.024–2019 Параметры эллиптических кривых для криптографических алгоритмов и протоколов.

ИНФОРМАЦИЯ О ПРОЕКТЕ
====================

Интернет-сайт проекта http://libakrypt.org

Исходные коды утилиты и библиотеки:
https://github.com/axelkenzo/libakrypt-0.x

АВТОРЫ
======

Axel Kenzo & The Company Of Belles Lettres (с) 2014 - 2020.
