# Инструкция по сборке и установке #

## Получение исходных текстов библиотеки ##

Cтабильная версия библиотеки распространяется в виде архива с исходными текстами,
в котором x обозначает старший номер релиза, а y младший номер релиза.


    libakrypt-0.x.y.tar.bz2


Для разархивирования необходимо выполнить команду


    tar -xjvf libakrypt-0.x.y.tar.bz2


Эту команду, как и все последующие, необходимо выполнять в консоли от имени обычного пользователя.
После разархивирования должен появиться корневой каталог `libakrypt-0.x`, содержащий следующие подкаталоги

* `aktool` - консольная утилита, использующая функции внешнего интерфейса,
* `cmake`  - управляющие файлы для системы сборки библиотеки,
* `doc` - документация,
* `examples` - примеры, использующие экспортируемые библиотекой функции (функции внешнего интерфейса),
* `source` - исходные тексты библиотеки,
* `tests` - тестовые примеры, использующие не экспортируемые функции (функции внутреннего интерфейса).

Пользователи `Windows` могут воспользоваться для разархивирования исходных 
текстов программой [7-Zip](https://www.7-zip.org/).

Последняя, возможно не самая стабильная версия исходных текстов библиотеки
может быть получена из репозитория на сайте [github.com](http://github.com/axelkenzo/libakrypt-0.x).
Для клонирования репозитория необходимо выполнить из командной строки следующую команду.


    git clone https://github.com/axelkenzo/libakrypt-0.x


Необходимая документация и описание процедуры установки распределенной
системы управления версиями `git` могут быть найдены на сайте [git-scm.org](http://git-scm.com).

## Система сборки ##

Системой сборки для библиотеки `libakrypt` является [cmake](https://cmake.org").
Система сборки не зависит от используемой операционной системы.
Необходимый набор программ и утилит может быть скачан с официального сайта программы.

Установка `cmake`, как правило, доступна для стандартных средств операционной системы.
Например, в Debian или Ubuntu, установка `cmake` может быть выполнена следующей командой


    apt install cmake


Данная команда должна выполняться с правами суперпользователя.


## Зависимости библиотеки ##

Библиотека `libakrypt` может реализовывать многопоточное выполнение некоторых криптографических преобразований,
используя для этого базовый набор функций для работы с потоками, определяемый `POSIX Threads`.

В операционных системах семейства `Linux`, а также в `FreeBSD`,
эта функциональность входит в состав библиотеки `libc` и не требует дополнительных действий при установке.

В операционной системе `Windows` многопоточность может быть реализована различными способами, при этом
способ реализации определяется используемым для сборки библиотеки компилятором:

* в случае, если для сборки библиотеки используется компилятор `gcc`, входящий в состав
набора библиотек и программ `MinGW`, то доступ к многопоточной функциональности
предоставляется библиотекой `libc` из пакета `MinGW`.

* в случае, если для сборки библиотеки в операционной системе Windows
используется компилятор `Microsoft Visual C` (или любой другой,
поставляемый без библиотек, реализующих многопоточную функциональность),
то необходимо использование разделяемой
библиотеки `pthreads-w32` (POSIX Threads Library for Win32).
Перед сборкой библиотеки `libakrypt` Вам необходимо вручную установить заголовочные и библиотечные файлы
(`pthreadVC.lib` и `pthreadVC.dll`, а также заголовочные файлы `pthread.h`, `semaphore.h` и `sched.h`).
Подробное описание установки `pthreads-w32` может быть найдено
в документации по [pthreads-w32](http://sources.redhat.com/pthreads-win32).

Отметим, что по желанию пользователья при сборке,
или в случае отсутствия `POSIX Thereads` в операционной системе пользователя,
соответствующая функциональность может быть удалена из библиотеки `libakrypt` при сборке
(отсутствие потоков может быть востребовано при кросс-компиляции статического кода).

**TODO:** Одной из важных, но не первостепенных задач, стоящих перед авторами библиотеки,
является сборка минимальной и переносимой конфигурации библиотеки `libc`,
для полного исключения зависимостей при сборке приложений.


## Сборка в Unix ##

Основной средой разработки библиотеки `libakrypt` является Linux,
поэтому процесс сборки под Unix-like операционными системами является максимально простой процедурой.

### Сборка библиотеки по-умолчанию ###

После получения исходных кодов и их разархивации,
для сборки библиотеки выполните в консоли следующую последовательность команд.


    mkdir build
    cd build
    cmake -DCMAKE_C_FLAGS="-march=native" ../libakrypt-0.x
    make


В результате сборки, по-умолчанию, будут собраны как динамическая, так и статическая версии 
библиотеки --- `libakrypt.so` и `libakrypt.a`,
а также ряд примеров, использующих функции внешнего (экспортируемого библиотекой) интерфейса.

Для отключения сборки, например, статической версии 
необходимо использовать флаг `LIBAKRYPT_STATIC_LIB`, передаваемый `cmake`.


    cmake -DLIBAKRYPT_STATIC_LIB=OFF ../libakrypt-0.x


Аналогичным образом отключается сборка динамической версии библиотеки.


    cmake -DLIBAKRYPT_SHARED_LIB=OFF ../libakrypt-0.x



### Сборка различными компиляторами ###

Приведенная нами выше последовательность команд использует для сборки библиотеки
найденный `cmake` компилятор по-умолчанию -- в Linux это компилятор `gcc`, во FreeBSD и MacOS это `clang`.
Если Вы хотите использовать другой компилятор, то Вам необходимо использовать
при вызове `cmake` опцию `CMAKE_C_COMPILER`, в явном виде определяющую имя компилятора.

Так, следующий вызов позволит произвести сборку библиотеки с помощью компилятора `clang`.


    cmake -DCMAKE_C_COMPILER=clang ../libakrypt-0.x


Аналогично, следующий вызов позволит произвести сборку библиотеки с помощью компилятора `tcc` ([Tiny C Compiler](https://bellard.org/tcc/))


    cmake -DCMAKE_C_COMPILER=tcc ../libakrypt-0.x


Отметим, что через опцию `CMAKE_C_COMPILER` можно указывать только те компиляторы,
которые поддерживаются `cmake`.
Перечень поддерживаемых компиляторов можно найти в документации по `cmake`
(см. раздел cmake-compile-features, supported compilers).


## Сборка в Windows ##

Под управлением операционной системы семейства `Windows`
может быть собрана как статическая, так и динамическая версия библиотеки.
Далее мы описываем способ сборки библиотеки не использующий графические средства разработки.

### Сборка с использованием компилятора MSVC ###
На настоящий момент протестирована успешная сборка библиотеки с помощью компилятора MSVC версий 10 и старше.
Скачать бесплатно распространяемую среду сборки от `Microsoft` можно
скачать [здесь](https://visualstudio.microsoft.com/ru/vs/community).

Для сборки библиотеки и необходимо запустить командную строку `Visual Studio` и
создать каталог для сборки, например, выполнив команду


    mkdir build-msvc

Далее, необходимо перейти в созданный каталог и запустить `cmake` для конфигурации сборки.


    cmake -G "NMake Makefiles" -DCMAKE_C_FLAGS="-march=native" ../libakrypt-0.x

Флаг `-G` определяет имя механизма, используемого далее для сборки исходных текстов.
Сборка библиотеки и тестовых примеров выполняется следующей командой


    nmake

Указанный выше пример позволит создать 	как динамическую версию библиотеки (.dll),
так и статическую (.lib). Механизм отключения сборки либо статической, либо динамической версии библиотеки
аналогичен сказанному выше.


В случае компиляции библиотеки с поддержкой `POSIX Threads`
для запуска тестовых примеров необходимо, чтобы либо в созданном Вами каталоге `build-msvc`,
либо в стандартных путях, доступных операционной системе,
находился файл `pthreadVC2.dll`.
В случае, если данный файл не будет найден, операционная система выдаст соответствующее предупреждение.

### Сборка с использованием компилятора GCC ###
Для сборки компилятором `gcc` Вам необходимо установить набор программ из проекта `MinGW`
(Minimalist GNU for Windows).
Версия компилятора для 32-х битных систем может быть найдена на сайте [mingw.org](http://mingw.org/wiki).
Различные варианты 64-х битных версий компилятора GCC могут быть
найдены на сайте [mingw-w64.org](http://mingw-w64.org/doku.php).


Для сборки библиотеки в командной строке (используйте консоль MinSYS) выполнить следующую последовательность команд.


    mkdir build
    cd build
    cmake -G "MinGW Makefiles" -DCMAKE_C_FLAGS="-march=native" ../libakrypt-0.x
    mingw32-make.exe

Флаги для сборки статической версии библиотеки аналогичны указанным выше.

## Кросс-компиляция ##

Кросс-компиляция это процесс, в котором вы компилируете на одной машине программы,
которые будут выполняться на другой машине. Машина на которой происходит компиляция
называется хостом (host machine), а машина, для которой предназначается программа,
называется таргетом или целевой машиной (target machine).

Приведем пример кросс-компиляции для случая, когда в качестве хоста выступает машина с архитектурой
`x86` под управлением ОС `Linux`, а целевой машиной служат машины с архитектурой `mips` - как 32-х разрядная, так и 64-х разрядная.
Для начала необходимо установить соответствующий компилятор и программные средства для тестирования
(мы используем Debian).


    apt install gcc-mips-linux-gnu gcc-mips64-linux-gnuabi64
    apt install qemu-system-mips

В первой строке мы устанавливаем кросс-компилятор `gcc`,
а во второй - виртуальную машину `qemu`.

Теперь для сборки библиотеки и запуска тестовых примеров под архитектурой mips32r2 достаточно выполнить


    cmake -DCMAKE_C_COMPILER="mips-linux-gnu-gcc" 
                            -DCMAKE_C_FLAGS="-march=mips32r2" ../libakrypt-0.x/
    make
    qemu-mips-static -L /usr/mips-linux-gnu/ ./example-hello

В первой строке мы настраиваем сборку библиотеки с использованием компилятора `mips-linux-gnu-gcc`,
реализующего процесс генерации кода, исполняемого на mips32r2 архитектуре. В третьей строке
мы используем виртуальную машину для запуска скомпилированного кода.

Аналогично, для сборки библиотеки и запуска тестовых примеров под архитектурой mips64r2 достаточно выполнить

    cmake -DCMAKE_C_COMPILER=mips64-linux-gnuabi64-gcc 
                            -DCMAKE_C_FLAGS="-march=mips64r2" ../libakrypt-0.x/
    make
    qemu-mips64-static -L /usr/mips64-linux-gnuabi64/ ./test-bckey04


## Сборка тестовых примеров ##

Для проверки корректноcти сборки библиотеки (вне зависимости от компилятора или операционной системы) можно
собрать систему тестов, использующих не экспортируемые функции библиотеки (функции внутреннего интерфейса).
По умолчанию, данная возможность выключена. Для ее включения достаточно выполнить


    cmake -D LIBAKRYPT_INTERNAL_TESTS=ON ../libakrypt-0.x

При сборке библиотеки компилятор также соберет тестовые примеры,
используя для этого статическую версию библиотеки.
После этого, для запуска системы тестов под `Unix` достаточно запустить


    make test

В `Windows` необходимо будет запустить


    nmake test

или

    mingw32-make test

в зависимости от того, какой компилятор используется для сборки библиотеки и тестовых примеров.
После прохождения тестов будет выведена информация об общем числе успешно пройденных тестов,
а также времени их работы.

Также добавим, что вместе с библиотекой собирается пример

    example-hello

который также выполняет функции динамического тестирования работоспособности библиотеки,
включая анализ используемой архитектуры, доступных элементарных функций,
а также всех реализованных криптографических преобразований.


## Инсталляция библиотеки ##

В текущей версии библиотеки поддерживается процесс
инсталляции библиотеки только под Unix-like операционными системами.

По умолчанию предполагается, что библиотека будет установлена в каталог `/usr/local`.
Для изменения этого каталога
можно передать в `cmake` путь установки в явном виде. Например, следующий вызов позволяет
установить библиотеку в католог `/usr`.


    cmake -DCMAKE_INSTALL_PREFIX=/usr ../libakrypt-0.x


Для инсталляции библиотеки достаточно выполнить команду


    make install


**Внимание.** Команда инсталляции библиотеки должна выполняться с правами суперпользователя.

## Сборка документации ##

Для сборки полной документациии необходимо установить
программу [Doxygen](http://www.doxygen.org/index.html).
Также желательно установить LaTex и программу [pandoc](https://pandoc.org)


    apt install doxygen pandoc texlive-full


После этого станет возможным собрать документацию в форматах `html` и `pdf` с помощью
последовательности команд


    cmake -DLIBAKRYPT_DOC=ON ../libakrypt-0.x
    make doc


Созданная документация будет находиться в подкаталоге `doc` каталога, в котором выполняется сборка,
а также будет упакована в архив `libakrypt-doc-0.x.y.tar.bz2`,
расположенный в каталоге сборки библиотеки.

Дополнительно,
если у Вас установлена программа qhelpgenerator (входит в стандартную установку библиотеки Qt),
то указанный вызов также создаст документацию в формате `qch`
(в каталоге сборки должен появиться файл `libakrypt-doc-0.x.y.qch`).


## Перечень опций для сборки библиотеки ##

В заключение, приведем перечень опций, которые могут передаваться в `cmake` для настройки и уточнения значений
параметров сборки библиотеки.
Данные параметры определены только для библиотеки `libakrypt` и дополняют существующие флаги `cmake`.

Применение опции  производится с помощью следующего вызова.


    cmake -D<имя опции>=<значение опции>  ../libakrypt-0.x


Вывести значения установленных опций можно с помощью вызова 


    cmake -L ../libakrypt-0.x

    
### LIBAKRYPT_AKTOOL ###

Опция `LIBAKRYPT_AKTOOL` определяется в `CMakeLists.txt` следующим образом


    option( LIBAKRYPT_AKTOOL "Build console aktool application" ON )

Опция указывает на необходимость сборки библиотеки вместе с консольной утилитой
`aktool`, предназначенной для выполнения различных криптографических преобразований с файлами
и иллюстрации возможностей библиотеки. В частности, утилита предоставляет возможность вычислять контрольные
суммы файлов, зашифровывать файлы, подписывать файлы с помощью электронной подписи.

Принимаемые значения: `ON`, `OFF`.

Значение по-умолчанию: `ON`.


### LIBAKRYPT_CONF ###

Опция `LIBAKRYPT_CONF` устанавливает каталог инсталляции и последующего
поиска конфигурационного файла `libakrypt.conf`, содержащего точные
значения технических и криптографических характеристик библиотеки.

Принимаемые значения: произвольная строка символов.

Значение по умолчанию: в Unix-подобных системах значением является каталог `/etc`,
для Windows-подобных систем значение не определено.

### LIBAKRYPT_CONST_CRYPTO_PARAMS ###
Опция `LIBAKRYPT_CONST_CRYPTO_PARAMS` определяется в `CMakeLists.txt` следующим образом


    option( LIBAKRYPT_CONST_CRYPTO_PARAMS 
            "Build library with const values of crypto parameters" OFF )

Опция указывает, можно ли после сборки библиотеки
контролировать значения технических и критографических характеристик библиотеки.

Если значение опции установлено в `ON`, то библиотека
компилируется без поддержки функций чтения технических характеристик, а файл,
определяемый значением опции `LIBAKRYPT_CONF`, не создается. При вычислениях используются константные
значения, жестко зашитые в исходные тексты библиотеки.

Принимаемые значения: `ON`, `OFF`.

Значение по-умолчанию: `OFF`.


### LIBAKRYPT_CRYPTO_FUNCTIONS ###

Опция `LIBAKRYPT_CRYPTO_FUNCTIONS` определяется в `CMakeLists.txt` следующим образом


    option( LIBAKRYPT_CRYPTO_FUNCTIONS 
                              "Build library with crypto functions" ON )

Опция указывает необходимость сборки библиотеки с использованием криптографических преобразований.

В случае, когда опция принимает значение `OFF`, собранная библиотека
содержит только базовые преобразования:

 * реализацию арифметических операций с большими целыми числами,
 * реализацию арифметических операций с элементами полей Галуа $\mathbb F_{2^n}$,
 * реализацию операций сложения и удвоения, а также вычисления кратной точки на эллиптической кривой,
 * функции чтения/записи значений технических характеристик библиотеки.

Какие-либо криптографические преобразования в этом случае не поддерживаются.

Принимаемые значения: `ON`, `OFF`.

Значение по-умолчанию: `ON`.


### LIBAKRYPT_INSTALL_HEADERS ###

Опция `LIBAKRYPT_INSTALL_HEADERS` определяется в `CMakeLists.txt` следующим образом


    option( LIBAKRYPT_INSTALL_HEADERS 
           "Install development headers with non-export functions" OFF )

Опция позволяет при инсталляции библиотеки также установить заголовочные файлы
для функций внутреннего интерфейса библиотеки.

Принимаемые значения: `ON`, `OFF`.

Значение по-умолчанию: `OFF`.


### LIBAKRYPT_INTERNAL_TESTS ###

Опция `LIBAKRYPT_INTERNAL_TESTS` определяется в `CMakeLists.txt` следующим образом


    option( LIBAKRYPT_INTERNAL_TESTS 
     "Build collection of internal tests using non-export functions" OFF )

Опция добавляет к сборке библиотеки сборку нескольких тестовых примеров,
напрямую использующих неэкспортируемые функции библиотеки. Данная возможность доступна только при одновременной сборке
статической версии библиотеки.
После сборки тестовых примеров становится доступной команда, запускающая процесс тестирования.


    make test

Принимаемые значения: `ON`, `OFF`.

Значение по-умолчанию: `OFF`.


### LIBAKRYPT_GMP_TESTS ###

Опция `LIBAKRYPT_GMP_TESTS` определяется в `CMakeLists.txt` следующим образом


    option( LIBAKRYPT_GMP_TESTS  
                    "Build comparison tests for gmp and libakrypt" OFF )

Опция добавляет к сборке библиотеки сборку нескольких тестовых примеров,
проводящих вычисления с использованием библиотеки [libgmp](http://gmplib.org).
Данные тестовые примеры сравнивают корректность реализации
арифметических операций с большими целыми числами
для библиотеки `libakrypt` и библиотеки `libgmp`.
После сборки тестовых примеров становится доступной команда, запускающая процесс тестирования.


    make test

Принимаемые значения: `ON`, `OFF`.

Значение по-умолчанию: `OFF`.


### LIBAKRYPT_DOC ###

Опция `LIBAKRYPT_DOC` определяется в `CMakeLists.txt` следующим образом


    option( LIBAKRYPT_DOC "Build documentation" OFF )

Опция добавляет возможность сборки документации в форматах `html` и `pdf`.
После сборки библиотеки становится доступной команда, запускающая процесс
сборки документации.


    make doc

Принимаемые значения: `ON`, `OFF`.

Значение по-умолчанию: `OFF`.


### LIBAKRYPT_SHARED_LIB ###

Опция `LIBAKRYPT_SHARED_LIB` определяется в `CMakeLists.txt` следующим образом


    option( LIBAKRYPT_SHARED_LIB "Build the shared library" ON )

Опция устанавливает, нужно ли собирать динамическую версию библиотеки.
Отметим, что одновременно может быть собрана только одна версия библиотеки либо динамическая, либо статическая.

Принимаемые значения: `ON`, `OFF`.

Значение по-умолчанию: `ON`.



### LIBAKRYPT_STATIC_LIB ###

Опция `LIBAKRYPT_STATIC_LIB` определяется в `CMakeLists.txt` следующим образом


    option( LIBAKRYPT_STATIC_LIB "Build the static library" OFF )

Опция устанавливает, нужно ли собирать статическую версию библиотеки.
Отметим, что одновременно может быть собрана только одна версия библиотеки либо динамическая, либо статическая.

Принимаемые значения: `ON`, `OFF`.

Значение по-умолчанию: `ON`.

