/* ----------------------------------------------------------------------------------------------- */
/*  Copyright (c) 2014 - 2019 by Axel Kenzo, axelkenzo@mail.ru                                     */
/*                                                                                                 */
/*  Файл ak_tools.h                                                                                */
/*  - содержит описания служебных функций и переменных, не экспортируемых за пределы библиотеки    */
/* ----------------------------------------------------------------------------------------------- */
 #ifndef    __AK_TOOLS_H__
 #define    __AK_TOOLS_H__

/* ----------------------------------------------------------------------------------------------- */
 #include <ak_random.h>

/* ----------------------------------------------------------------------------------------------- */
#ifdef LIBAKRYPT_HAVE_WINDOWS_H
 #include <windows.h>
#endif

/* ----------------------------------------------------------------------------------------------- */
/*! \brief Структура данных для хранения дескриптора и параметров файла. */
 typedef struct file {
#ifdef LIBAKRYPT_HAVE_WINDOWS_H
 /*! \brief Дескриптор файла для операционной системы Windows. */
  HANDLE hFile;
#else
 /*! \brief Дескриптор файла. */
  int fd;
#endif
 /*! \brief Размер файла. */
  ak_int64 size;
 /*! \brief Размер блока для оптимального чтения с жесткого диска. */
  ak_int64 blksize;
 } *ak_file;

/* ----------------------------------------------------------------------------------------------- */
/*! \brief Функция выделения динамической памяти. */
 ak_pointer ak_libakrypt_aligned_malloc( size_t );

/* ----------------------------------------------------------------------------------------------- */
/*! \brief Функция очистки памяти. */
 int ak_ptr_context_wipe( ak_pointer , size_t , ak_random );
/*! \brief Функция сравнения двух массивов данных и вывода информации в систему аудита. */
 bool_t ak_ptr_is_equal_with_log( ak_const_pointer, ak_const_pointer , const size_t );

/* ----------------------------------------------------------------------------------------------- */
/*! \brief Функция открывает заданный файл на чтение. */
 int ak_file_open_to_read( ak_file , const char * ); 
/*! \brief Функция создает файл с правами на запись. */
 int ak_file_create_to_write( ak_file , const char * );
/*! \brief Функция закрывает файл с заданным дескриптором. */
 int ak_file_close( ak_file );
/*! \brief Функция считывает заданное количество байт из файла. */
 ssize_t ak_file_read( ak_file , ak_pointer , size_t );
/*! \brief Функция записывает заданное количество байт в файл. */
 ssize_t ak_file_write( ak_file , ak_const_pointer , size_t );
/*! \brief Функция записывает в файл строку символов. */
 ssize_t ak_file_printf( ak_file , const char * , ... );

/* ----------------------------------------------------------------------------------------------- */
/*! \brief Функция устанавливает значение опции с заданным именем. */
 int ak_libakrypt_set_option( const char *name, const ak_int64 value );
/*! \brief Функция возвращает значение опции с заданным именем. */
 ak_int64 ak_libakrypt_get_option( const char *name );
/*! \brief Вывод в логгер текущих значений опций библиотеки. */
 void ak_libakrypt_log_options( void );

/* ----------------------------------------------------------------------------------------------- */
#ifndef LIBAKRYPT_CONST_CRYPTO_PARAMS
/*! \brief Функция считывает настройки (параметры) библиотеки из файла libakrypt.conf */
 bool_t ak_libakrypt_load_options( void );
#endif

/* ----------------------------------------------------------------------------------------------- */
/*! \brief Функция кодирует три байта информации в формат base64.  */
 void ak_base64_encodeblock( ak_uint8 *, ak_uint8 *, int );

#endif
/* ----------------------------------------------------------------------------------------------- */
/*                                                                                 ak_libakrypt.h  */
/* ----------------------------------------------------------------------------------------------- */
