/* Тестовый пример для оценки скорости операции умножения в
   полях характеристики два
   Пример использует неэкспортируемые функции.

   test-gf2n.c
*/

 #include <time.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <ak_gf2n.h>

 static ak_uint32 iteration_count = 100000;
 static ak_uint8 alpha[64]  = {
   0xa1, 0xb2, 0xc4, 0x31, 0xa0, 0xff, 0xac, 0x13, 0x11, 0x10, 0xaa, 0x12, 0x73, 0x53, 0x4a, 0x01,
   0xC6, 0x00, 0xFD, 0x9D, 0xD0, 0x49, 0xCF, 0x8A, 0xBD, 0x2F, 0x5B, 0x32, 0xE8, 0x40, 0xD2, 0xCB,
   0x0E, 0x41, 0xEA, 0x44, 0xDE, 0x1C, 0x15, 0x5D, 0xCD, 0x88, 0xDC, 0x84, 0xFE, 0x58, 0xA8, 0x55,
   0xff, 0xff, 0x54, 0x36, 0x27, 0x1c, 0x21, 0xff, 0x67, 0x0c, 0x09, 0x81, 0x2f, 0xaa, 0x1a, 0xc6 };

 static ak_uint8 beta[64]  = {
   0x01, 0x0a, 0x21, 0x11, 0xad, 0xc7, 0x58, 0x75, 0x00, 0x21, 0x21, 0x21, 0x11, 0x1f, 0xff, 0xff,
   0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xF0, 0xFF, 0xFF, 0xFF, 0xFF, 0xea, 0x12, 0xcc, 0xa3,
   0x21, 0xaa, 0xa1, 0x17, 0x18, 0x17, 0xaa, 0xcf, 0xf3, 0xe2, 0xca, 0xdf, 0xe4, 0x30, 0x98, 0x88,
   0xac, 0x12, 0x12, 0x11, 0x12, 0x20, 0x98, 0x17, 0x2a, 0xaa, 0x34, 0x38, 0x47, 0x83, 0x47, 0xaa };

 static ak_uint8 gamma[64] = { 0x0LL, 0x0LL, 0x0LL, 0x0LL, 0x0LL, 0x0LL, 0x0LL, 0x0LL };

#ifdef LIBAKRYPT_HAVE_BUILTIN_CLMULEPI64
 static ak_uint8 delta[64] = { 0x0LL, 0x0LL, 0x0LL, 0x0LL, 0x0LL, 0x0LL, 0x0LL, 0x0LL };
#endif

/* функция тестирования */
 void gftest( void (func)( ak_pointer , ak_pointer , ak_pointer ),
                                                 char *str, size_t n, ak_uint8 *buffer )
{
  time_t time;
  ak_uint32 i = 0;

    time = clock();
    for( i = 0; i < iteration_count; i++ ) func( buffer, alpha, beta );
    time = clock() - time;

    printf(" GF(2^%u): ", (unsigned int)n );
    for( i = 0; i < n/8; i++ ) printf("%02x", buffer[i] );
    printf(" (%s, %f sec)\n", str, (double)time / (double)CLOCKS_PER_SEC );
}

 int main( void )
{
   ak_uint8 t64[8] =
     { 0x12, 0xf0, 0x98, 0x16, 0x60, 0xbf, 0x82, 0x07 };

   ak_uint8 t128[16] =
     { 0x53, 0x41, 0x10, 0x77, 0xe6, 0xd6, 0xf5, 0x1a, 0xe6, 0xe4, 0xe5, 0x0a, 0xcf, 0x4a, 0x42, 0x7a };

   ak_uint8 t256[32] =
     { 0x2b, 0xea, 0x76, 0xfa, 0x99, 0xaf, 0x07, 0x3a, 0x46, 0x9e, 0x1c, 0x5d, 0x56, 0xb8, 0x9b, 0x9e,
       0x19, 0xc3, 0xe2, 0x36, 0x14, 0x6e, 0x59, 0x99, 0x7d, 0x6a, 0xc5, 0x58, 0xbb, 0x9e, 0x5b, 0x12 };

   ak_uint8 t512[64] =
     { 0x40, 0x39, 0xd4, 0x68, 0xd2, 0xd9, 0x35, 0xc1, 0x4f, 0x22, 0x42, 0xb6, 0xe3, 0x9d, 0xb3, 0xb5,
       0xd5, 0x99, 0x95, 0xc1, 0xe7, 0xa0, 0x11, 0x87, 0xf6, 0xf7, 0xa3, 0xd7, 0xe3, 0xc2, 0x6a, 0xc3,
       0x19, 0x12, 0xf4, 0xc2, 0x4e, 0x1d, 0x64, 0xfe, 0x62, 0xec, 0x44, 0xad, 0x48, 0xd8, 0xa4, 0x6b,
       0x7a, 0x9e, 0xf8, 0xe4, 0xab, 0x7f, 0x7b, 0x3b, 0x47, 0x95, 0x18, 0x3d, 0xf6, 0x73, 0x1c, 0x1e };


   gftest( ak_gf64_mul_uint64, "ak_gf64_mul_uint64", 64, gamma );
 #ifdef LIBAKRYPT_HAVE_BUILTIN_CLMULEPI64
   gftest( ak_gf64_mul_pcmulqdq, "ak_gf64_mul_pcmulqdq", 64, delta );
   printf(" dual test is ");
   if( ak_ptr_is_equal( gamma, delta, 8 )) printf("Ok\n");
     else { printf("Wrong\n"); return EXIT_FAILURE; }
 #endif
   printf(" const value test is ");
   if( ak_ptr_is_equal( gamma, t64, sizeof( t64 ))) printf("Ok\n\n");
     else { printf("Wrong\n\n"); return EXIT_FAILURE; }


   gftest( ak_gf128_mul_uint64, "ak_gf128_mul_uint64", 128, gamma );
 #ifdef LIBAKRYPT_HAVE_BUILTIN_CLMULEPI64
   gftest( ak_gf128_mul_pcmulqdq, "ak_gf128_mul_pcmulqdq", 128, delta );
   printf(" dual test is ");
   if( ak_ptr_is_equal( gamma, delta, 16 )) printf("Ok\n");
     else { printf("Wrong\n"); return EXIT_FAILURE; }
 #endif
   printf(" const value test is ");
   if( ak_ptr_is_equal( gamma, t128, sizeof( t128 ))) printf("Ok\n\n");
     else { printf("Wrong\n\n"); return EXIT_FAILURE; }


   gftest( ak_gf256_mul_uint64, "ak_gf256_mul_uint64", 256, gamma );
 #ifdef LIBAKRYPT_HAVE_BUILTIN_CLMULEPI64
   gftest( ak_gf256_mul_pcmulqdq, "ak_gf256_mul_pcmulqdq", 256, delta );
   printf(" dual test is ");
   if( ak_ptr_is_equal( gamma, delta, 32 )) printf("Ok\n");
     else { printf("Wrong\n"); return EXIT_FAILURE; }
 #endif
   printf(" const value test is ");
   if( ak_ptr_is_equal( gamma, t256, sizeof( t256 ))) printf("Ok\n\n");
     else { printf("Wrong\n\n"); return EXIT_FAILURE; }


   gftest( ak_gf512_mul_uint64, "", 512, gamma );
 #ifdef LIBAKRYPT_HAVE_BUILTIN_CLMULEPI64
   gftest( ak_gf512_mul_pcmulqdq, "", 512, delta );
   printf(" dual test is ");
   if( ak_ptr_is_equal( gamma, delta, 64 )) printf("Ok\n");
     else { printf("Wrong\n"); return EXIT_FAILURE; }
 #endif
   printf(" const value test is ");
   if( ak_ptr_is_equal( gamma, t512, 64 )) printf("Ok\n\n");
     else { printf("Wrong\n\n"); return EXIT_FAILURE; }

 return EXIT_SUCCESS;
}
