/* Пример иллюстрирует применение неэкспортируемых функций для чтения
   открытых ключей электронной подписи.
*/

 #include <stdio.h>
 #include <stdlib.h>

 #include <ak_sign.h>
 #include <ak_asn1_keys.h>

 const ak_uint8 reqedw[205] = {
  0x30, 0x81, 0xca, 0x30, 0x79, 0x02, 0x01, 0x00, 0x30, 0x12, 0x31, 0x10, 0x30, 0x0e, 0x06, 0x03,
  0x55, 0x04, 0x03, 0x13, 0x07, 0x45, 0x78, 0x61, 0x6d, 0x70, 0x6c, 0x65, 0x30, 0x5e, 0x30, 0x17,
  0x06, 0x08, 0x2a, 0x85, 0x03, 0x07, 0x01, 0x01, 0x01, 0x01, 0x30, 0x0b, 0x06, 0x09, 0x2a, 0x85,
  0x03, 0x07, 0x01, 0x02, 0x01, 0x01, 0x01, 0x03, 0x43, 0x00, 0x04, 0x40, 0x74, 0x27, 0x95, 0xd4,
  0xbe, 0xe8, 0x84, 0xdd, 0xf2, 0x85, 0x0f, 0xec, 0x03, 0xea, 0x3f, 0xaf, 0x18, 0x44, 0xe0, 0x1d,
  0x9d, 0xa6, 0x0b, 0x64, 0x50, 0x93, 0xa5, 0x5e, 0x26, 0xdf, 0xc3, 0x99, 0x78, 0xf5, 0x96, 0xcf,
  0x4d, 0x4d, 0x0c, 0x6c, 0xf1, 0xd1, 0x89, 0x43, 0xd9, 0x44, 0x93, 0xd1, 0x6b, 0x9e, 0xc0, 0xa1,
  0x6d, 0x51, 0x2d, 0x2e, 0x12, 0x7c, 0xc4, 0x69, 0x1a, 0x63, 0x18, 0xe2, 0xa0, 0x00, 0x30, 0x0a,
  0x06, 0x08, 0x2a, 0x85, 0x03, 0x07, 0x01, 0x01, 0x03, 0x02, 0x03, 0x41, 0x00, 0x1b, 0xdc, 0x2a,
  0x13, 0x17, 0x67, 0x9b, 0x66, 0x23, 0x2f, 0x63, 0xea, 0x16, 0xff, 0x7c, 0x64, 0xcc, 0xaa, 0xb9,
  0xad, 0x85, 0x5f, 0xc6, 0xe1, 0x80, 0x91, 0x66, 0x1d, 0xb7, 0x9d, 0x48, 0x12, 0x1d, 0x0e, 0x1d,
  0xa5, 0xbe, 0x34, 0x7c, 0x6f, 0x1b, 0x52, 0x56, 0xc7, 0xae, 0xac, 0x20, 0x0a, 0xd6, 0x4a, 0xc7,
  0x7a, 0x6f, 0x5b, 0x3a, 0x0e, 0x09, 0x73, 0x18, 0xe7, 0xae, 0x6e, 0xe7, 0x69
 };

 int main( int argc, char *argv[] )
{
  struct verifykey vk;
  int ecode = EXIT_SUCCESS;
  char *fname = "reqedw.p10";

 /* эксперимент по чтению открытого ключа из запроса на сертификат
    если имя файла в командной строке не указано, то используется
    тестовый запрос из Р 1323565.1.023-2018 */
  if( argc < 2 ) {
    FILE *fp = fopen( fname, "wb" );
    fwrite( reqedw, sizeof( reqedw ), 1, fp );
    fclose( fp );
  } else fname = argv[1];

  ak_libakrypt_create( ak_function_log_stderr );
  if( ak_verifykey_context_import_from_request( &vk, fname ) == ak_error_ok ) {

    printf("algorithm: %s (%s)\n", vk.oid->info.name[0], vk.oid->info.id[0] );
    printf("loaded curve:\n p = %s\n",
                ak_ptr_to_hexstr( vk.wc->p, sizeof( ak_uint64 )*vk.wc->size, ak_true ));
    printf(" a = %s (in Montgomery form)\n",
                ak_ptr_to_hexstr( vk.wc->a, sizeof( ak_uint64 )*vk.wc->size, ak_true ));
    printf(" b = %s (in Montgomery form)\n\n",
                ak_ptr_to_hexstr( vk.wc->b, sizeof( ak_uint64 )*vk.wc->size, ak_true ));
    if( vk.flags == ak_key_flag_undefined ) printf(" key is undefined\n");
     else {
       printf("public key:\n P.x: %s\n", ak_mpzn_to_hexstr( vk.qpoint.x, vk.wc->size ));
       printf(" P.y: %s\n", ak_mpzn_to_hexstr( vk.qpoint.y, vk.wc->size ));
       if( vk.name != NULL ) {
         printf("name:\n");
         ak_tlv_context_print( vk.name, stdout );
       }
     }
    ak_verifykey_context_destroy( &vk );
  }
   else {
          fprintf( stdout, "incorrect creation of verify context\n");
          ecode = EXIT_FAILURE;
        }

  ak_libakrypt_destroy();
 return ecode;
}
